@@ -57,6 +57,42 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 self.assertEqual(content1, content2)
                 r.close()
 
+    def test_client_HTTP_200_OK_METHOD_with_session(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            for meth in ('get', 'post', 'put', 'delete', 'head'):
+                host = httpd.url().split('://', 1)[-1]
+                httpclient = client.HttpClient(
+                    host, loop=self.loop, session=True)
+
+                r = self.loop.run_until_complete(
+                    httpclient.request(meth, '/method/%s' % meth))
+                content1 = self.loop.run_until_complete(r.read())
+                content2 = self.loop.run_until_complete(r.read())
+                content = content1.decode()
+
+                self.assertEqual(r.status, 200)
+                self.assertIn('"method": "%s"' % meth.upper(), content)
+                self.assertEqual(content1, content2)
+                r.close()
+
+    def test_client_HTTP_200_OK_METHOD_no_resolve(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            for meth in ('get', 'post', 'put', 'delete', 'head'):
+                host = httpd.url().split('://', 1)[-1]
+                httpclient = client.HttpClient(
+                    host, loop=self.loop, resolve=False)
+
+                r = self.loop.run_until_complete(
+                    httpclient.request(meth, '/method/%s' % meth))
+                content1 = self.loop.run_until_complete(r.read())
+                content2 = self.loop.run_until_complete(r.read())
+                content = content1.decode()
+
+                self.assertEqual(r.status, 200)
+                self.assertIn('"method": "%s"' % meth.upper(), content)
+                self.assertEqual(content1, content2)
+                r.close()
+
     def test_client_defaults_HTTP_200_OK_METHOD(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             host = httpd.url().split('://', 1)[-1]
@@ -498,7 +534,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             httpd['close'] = True
             self.assertRaises(
-                aiohttp.HttpException,
+                aiohttp.ClientConnectionError,
                 self.loop.run_until_complete,
                 client.request('get', httpd.url('method', 'get'),
                                loop=self.loop))
