@@ -328,14 +328,42 @@ class HttpRequestTests(unittest.TestCase):
         self.assertTrue(inspect.isgenerator(req.body))
         self.assertEqual(req.headers['transfer-encoding'], 'chunked')
 
-        req.send(self.transport)
-        self.loop.run_until_complete(req._writer)
+        resp = req.send(self.transport)
+        self.assertIsInstance(req._writer, asyncio.Future)
+        self.loop.run_until_complete(resp.wait_for_close())
+        self.assertIsNone(req._writer)
         self.assertEqual(
             self.transport.write.mock_calls[-3:],
             [unittest.mock.call(b'binary data result'),
              unittest.mock.call(b'\r\n'),
              unittest.mock.call(b'0\r\n\r\n')])
 
+    def test_data_stream_exc(self):
+        fut = asyncio.Future(loop=self.loop)
+
+        def gen():
+            yield b'binary data'
+            yield from fut
+            return b' result'
+
+        req = HttpRequest(
+            'POST', 'http://python.org/', data=gen(), loop=self.loop)
+        self.assertTrue(req.chunked)
+        self.assertTrue(inspect.isgenerator(req.body))
+        self.assertEqual(req.headers['transfer-encoding'], 'chunked')
+
+        @asyncio.coroutine
+        def exc():
+            yield from asyncio.sleep(0.01, loop=self.loop)
+            fut.set_exception(ValueError)
+
+        asyncio.async(exc(), loop=self.loop)
+
+        req.send(self.transport)
+        self.assertRaises(
+            ValueError, self.loop.run_until_complete, req._writer)
+        self.assertRaises(self.transport.close.called)
+
     def test_data_stream_not_bytes(self):
         @asyncio.coroutine
         def gen():
@@ -435,6 +463,19 @@ class HttpClientTests(unittest.TestCase):
         c = HttpClient([('localhost', 1000)])
         self.assertIs(c._loop, asyncio.get_event_loop.return_value)
 
+    def test_cleanup_resolved_hosts(self):
+        loop = unittest.mock.Mock()
+        c = HttpClient('localhost:8080', loop=loop, resolve=True)
+        c._resolved_hosts[('localhost', 123)] = object()
+        loop.call_later.assert_called_with(
+            c._resolve_timeout, c._cleanup_resolved_host)
+        loop.reset_mock()
+
+        c._cleanup_resolved_host()
+        self.assertFalse(bool(c._resolved_hosts))
+        loop.call_later.assert_called_with(
+            c._resolve_timeout, c._cleanup_resolved_host)
+
     @unittest.mock.patch('aiohttp.client.asyncio')
     def test_resurrect_failed(self, asyncio):
         now = int(time.time())
@@ -455,7 +496,8 @@ class HttpClientTests(unittest.TestCase):
     def test_resurrect_failed_all(self, asyncio):
         now = int(time.time())
 
-        c = HttpClient([('localhost', 1000), ('localhost', 1000)])
+        c = HttpClient(
+            [('localhost', 1000), ('localhost', 1000)], resolve=False)
         c._hosts = []
         c._failed.append((('localhost', 1000), now - 10))
         c._failed.append((('localhost', 1001), now - 10))
